
# q-expansion of newform 7605.2.a.h, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7605
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [1], [0], [4], [0], [-2], [4], [-8], [2], [8], [-6], [-6], [-4], [-8], [-6], [-12], [-2], [4], [0], [6], [16], [-4], [10], [-18], [-6], [-8], [-12], [2], [-2], [-16], [12], [-6], [-20], [-10], [0], [-2], [20], [16], [2], [12], [22], [-16], [14], [6], [-8], [20], [24], [12], [-22], [-26], [-24], [14], [4], [-18], [24], [-14], [8], [-10], [-22], [-20], [6], [4], [-24], [26], [30], [-20], [-14], [4], [-14], [-14], [16], [-16], [22], [-36], [24], [-6], [-14], [-30], [38], [-20], [10], [8], [18], [-24], [4], [18], [-26], [-18], [-8], [-4], [8], [-16], [20], [-28], [-24], [30], [22], [-20], [-14], [36], [-18], [-4], [38], [44], [-18], [-12], [-30], [-40], [-38], [32], [-6], [-38], [-4], [0], [30], [-28], [-24], [-30], [-36], [-38], [34], [10], [4], [36], [34], [10], [48], [-24], [-30], [4], [0], [0], [-42], [10], [30], [6], [4], [-46], [-42], [-20], [22], [-24], [-12], [-34], [16], [42], [-10], [-4], [-40], [50], [14], [36], [-24], [-4], [-32], [-40], [18], [-22], [46], [12], [-10], [16], [36], [-30], [0], [0], [38], [-50], [-6], [-28], [-14], [24], [38], [-32], [6], [-20], [38], [-8], [46], [8], [-36], [-26], [-6], [24], [10], [30], [-28], [22], [8], [2], [20], [-12], [-66], [12], [-58], [-18], [62], [18], [-40], [14], [-48], [42], [-30], [-28], [18], [-56], [28], [-6], [-4], [-14], [42], [56], [-12], [0], [22], [-48], [-18], [48], [30], [38], [32], [-46], [-24], [-68], [-10], [-42], [48], [56], [4], [46], [20], [40], [-42], [-20], [40], [14], [54], [4], [40], [60], [-20], [-24], [34], [34], [72], [8], [60], [-20], [64], [2], [50], [-16], [74], [34], [-52], [-42], [-20], [42], [22], [-80], [60], [42], [-34], [-14], [4], [78], [10], [-4], [74], [-18], [20], [22], [-16], [50], [0], [-60], [18], [-74], [44], [-64], [16], [-24], [42], [-20], [-16], [-14], [22], [-64], [62], [18], [12], [-22], [-60], [46], [-30], [-64], [-6], [20], [-76], [42], [14], [64], [-52], [28], [-50], [4], [-18], [0], [-26], [-48], [22], [62], [36], [-48], [42], [60], [-8], [46], [-18], [4], [-10], [-30], [8], [-38], [50], [36], [76], [48], [42], [2], [50], [-32], [60], [-84], [36], [-46], [-14], [-10], [-32], [-22], [-42], [70], [-40], [-66], [60], [-90], [-20], [-72], [-6], [-76], [-58], [-50], [-32], [-10], [-10], [-56], [20], [-2], [48], [26], [54], [16], [36], [-44], [-54], [-82], [-32], [-38], [12], [-4], [56], [42], [-40], [-34], [-36], [-64], [-50], [-50], [-38], [30], [58], [-16], [50], [-60], [-80], [-40], [22], [12], [96], [30], [6], [-28], [52], [32], [-70], [8], [-74], [-52], [86], [-46], [46], [-8], [10], [70], [40], [82], [-62], [4], [-36], [-14], [-6], [-60], [-44], [-22], [50], [24], [-56], [-30], [24], [2], [26], [40], [-108], [70], [-18], [-4], [10], [-52], [-40]]
